@@ -1,9 +1,12 @@
 #ifndef _RAR_ERRHANDLER_
 #define _RAR_ERRHANDLER_
 
+#if (defined(GUI) || !defined(_WIN_32)) && !defined(SFX_MODULE) || defined(RARDLL)
+#define ALLOW_EXCEPTIONS
+#endif
 
 enum { SUCCESS,WARNING,FATAL_ERROR,CRC_ERROR,LOCK_ERROR,WRITE_ERROR,
-       OPEN_ERROR,USER_ERROR,MEMORY_ERROR,USER_BREAK=255};
+       OPEN_ERROR,USER_ERROR,MEMORY_ERROR,CREATE_ERROR,USER_BREAK=255};
 
 class ErrorHandler
 {
@@ -39,6 +42,7 @@ class ErrorHandler
     void Throw(int Code);
     void SetSilent(bool Mode) {Silent=Mode;};
     void SetShutdown(bool Mode) {DoShutdown=Mode;};
+    void SysErrMsg();
 };
 
 #endif
