@@ -3,6 +3,7 @@
 CmdExtract::CmdExtract():DataIO(NULL)
 {
   TotalFileCount=0;
+  *Password=0;
   Unp=new Unpack(&DataIO);
   Unp->Init(NULL);
 }
@@ -11,6 +12,7 @@ CmdExtract::CmdExtract():DataIO(NULL)
 CmdExtract::~CmdExtract()
 {
   delete Unp;
+  memset(Password,0,sizeof(Password));
 }
 
 
@@ -31,7 +33,7 @@ void CmdExtract::DoExtract(CommandData *Cmd)
       DataIO.ProcessedArcSize+=FD.Size;
   }
 
-  if (TotalFileCount==0)
+  if (TotalFileCount==0 && *Cmd->Command!='I')
   {
     mprintf(St(MExtrNoFiles));
     ErrHandler.SetErrorCode(WARNING);
@@ -59,8 +61,9 @@ void CmdExtract::ExtractArchiveInit(CommandData *Cmd,Archive &Arc)
   FirstFile=true;
 #endif
 
-  strcpy(Password,Cmd->Password);
-  PasswordAll=(*Password!=0);
+  if (*Cmd->Password!=0)
+    strcpy(Password,Cmd->Password);
+  PasswordAll=(*Cmd->Password!=0);
 
   DataIO.UnpVolume=false;
 
@@ -75,7 +78,10 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
 {
   Archive Arc(Cmd);
   if (!Arc.WOpen(ArcName,ArcNameW))
+  {
+    ErrHandler.SetErrorCode(OPEN_ERROR);
     return(EXTRACT_ARC_NEXT);
+  }
 
   if (!Arc.IsArchive(true))
   {
@@ -127,7 +133,6 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
       else
         break;
   }
-  memset(Password,0,sizeof(Password));
   return(EXTRACT_ARC_NEXT);
 }
 
@@ -208,6 +213,15 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
     AllMatchesExact=false;
 
   bool WideName=(Arc.NewLhd.Flags & LHD_UNICODE);
+
+#ifdef _APPLE
+  if (WideName)
+  {
+    WideToUtf(Arc.NewLhd.FileNameW,ArcFileName,sizeof(ArcFileName));
+    WideName=false;
+  }
+#endif
+
   wchar *DestNameW=WideName ? DestFileNameW:NULL;
 
 #ifdef UNICODE_SUPPORTED
@@ -274,6 +288,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
   }
 #endif
   DataIO.UnpVolume=(Arc.NewLhd.Flags & LHD_SPLIT_AFTER);
+  DataIO.NextVolumeMissing=false;
 
   Arc.Seek(Arc.NextBlockPos-Arc.NewLhd.FullPackSize,SEEK_SET);
 
@@ -320,7 +335,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       }
 #if !defined(GUI) && !defined(SILENT)
       else
-        if (!PasswordAll && !Arc.Solid)
+        if (!PasswordAll && (!Arc.Solid || Arc.NewLhd.UnpVer>=20 && (Arc.NewLhd.Flags & LHD_SOLID)==0))
         {
           eprintf(St(MUseCurPsw),ArcFileName);
           switch(Cmd->AllYes ? 1:Ask(St(MYesNoAll)))
@@ -407,7 +422,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       struct FindData FD;
       if (FindFile::FastFind(DestFileName,DestNameW,&FD))
       {
-        if (FD.FileTime >= Arc.NewLhd.FileTime)
+        if (FD.mtime >= Arc.NewLhd.mtime)
           ExtrFile=false;
       }
       else
@@ -468,8 +483,16 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
           return(true);
         }
         MKDIR_CODE MDCode=MakeDir(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
-        if (MDCode!=MKDIR_SUCCESS && !FileExist(DestFileName,DestNameW))
+        bool DirExist=false;
+        if (MDCode!=MKDIR_SUCCESS)
         {
+          DirExist=FileExist(DestFileName,DestNameW);
+          if (DirExist && !IsDir(GetFileAttr(DestFileName,DestNameW)))
+          {
+            bool UserReject;
+            FileCreate(Cmd,NULL,DestFileName,DestNameW,Cmd->Overwrite,&UserReject,Arc.NewLhd.UnpSize,Arc.NewLhd.FileTime);
+            DirExist=false;
+          }
           CreatePath(DestFileName,DestNameW,true);
           MDCode=MakeDir(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
         }
@@ -482,7 +505,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
           PrevExtracted=true;
         }
         else
-          if (FileExist(DestFileName,DestNameW))
+          if (DirExist)
           {
             SetFileAttr(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
             PrevExtracted=true;
@@ -490,13 +513,14 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
           else
           {
             Log(Arc.FileName,St(MExtrErrMkDir),DestFileName);
+            ErrHandler.SysErrMsg();
 #ifdef RARDLL
             Cmd->DllError=ERAR_ECREATE;
 #endif
-            ErrHandler.SetErrorCode(WARNING);
+            ErrHandler.SetErrorCode(CREATE_ERROR);
           }
         if (PrevExtracted)
-          SetDirTime(DestFileName,Arc.NewLhd.FileTime);
+          SetDirTime(DestFileName,&Arc.NewLhd.mtime,&Arc.NewLhd.ctime,&Arc.NewLhd.atime);
         return(true);
       }
       else
@@ -516,7 +540,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
             if (!UserReject)
             {
               ErrHandler.CreateErrorMsg(DestFileName);
-              ErrHandler.SetErrorCode(WARNING);
+              ErrHandler.SetErrorCode(CREATE_ERROR);
               if (!IsNameUsable(DestFileName))
               {
                 Log(Arc.FileName,St(MCorrectingName));
@@ -658,9 +682,15 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
         {
           if (BrokenFile)
             CurFile.Truncate();
-          CurFile.SetOpenFileStat(Arc.NewLhd.FileTime);
+          CurFile.SetOpenFileStat(
+            Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.NewLhd.mtime,
+            Cmd->xctime==EXTTIME_NONE ? NULL:&Arc.NewLhd.ctime,
+            Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime);
           CurFile.Close();
-          CurFile.SetCloseFileStat(Arc.NewLhd.FileTime,Arc.NewLhd.FileAttr);
+          CurFile.SetCloseFileStat(
+            Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.NewLhd.mtime,
+            Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime,
+            Arc.NewLhd.FileAttr);
           PrevExtracted=true;
         }
       }
@@ -668,7 +698,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
   }
   if (ExactMatch)
     MatchedArgs++;
-  if (!Arc.IsOpened())
+  if (DataIO.NextVolumeMissing || !Arc.IsOpened())
     return(false);
   if (!ExtrFile)
     if (!Arc.Solid)
