@@ -33,17 +33,18 @@
 #define   MCHelpCmdF         "\n  f             Freshen files in archive"
 #define   MCHelpCmdI         "\n  i[par]=<str>  Find string in archives"
 #define   MCHelpCmdK         "\n  k             Lock archive"
-#define   MCHelpCmdL         "\n  l[t]          List archive [technical]"
+#define   MCHelpCmdL         "\n  l[t,b]        List archive [technical, bare]"
 #define   MCHelpCmdM         "\n  m[f]          Move to archive [files only]"
 #define   MCHelpCmdP         "\n  p             Print file to stdout"
 #define   MCHelpCmdR         "\n  r             Repair archive"
 #define   MCHelpCmdRC        "\n  rc            Reconstruct missing volumes"
+#define   MCHelpCmdRN        "\n  rn            Rename archived files"
 #define   MCHelpCmdRR        "\n  rr[N]         Add data recovery record"
 #define   MCHelpCmdRV        "\n  rv[N]         Create recovery volumes"
 #define   MCHelpCmdS         "\n  s[name|-]     Convert archive to or from SFX"
 #define   MCHelpCmdT         "\n  t             Test archive files"
 #define   MCHelpCmdU         "\n  u             Update files in archive"
-#define   MCHelpCmdV         "\n  v[t]          Verbosely list archive [technical]"
+#define   MCHelpCmdV         "\n  v[t,b]        Verbosely list archive [technical,bare]"
 #define   MCHelpCmdX         "\n  x             Extract files with full path"
 #define   MCHelpSw           "\n\n<Switches>"
 #define   MCHelpSwm          "\n  -             Stop switches scanning"
@@ -76,6 +77,7 @@
 #define   MCHelpSwIERR       "\n  ierr          Send all messages to stderr"
 #define   MCHelpSwILOG       "\n  ilog[name]    Log errors to file (registered versions only)"
 #define   MCHelpSwINUL       "\n  inul          Disable all messages"
+#define   MCHelpSwIOFF       "\n  ioff          Turn PC off after completing an operation"
 #define   MCHelpSwISND       "\n  isnd          Enable sound"
 #define   MCHelpSwK          "\n  k             Lock archive"
 #define   MCHelpSwKB         "\n  kb            Keep broken extracted files"
@@ -105,6 +107,7 @@
 #define   MCHelpSwTO         "\n  to<time>      Process files older than <time>"
 #define   MCHelpSwTA         "\n  ta<date>      Process files modified after <date> in YYYYMMDDHHMMSS format"
 #define   MCHelpSwTB         "\n  tb<date>      Process files modified before <date> in YYYYMMDDHHMMSS format"
+#define   MCHelpSwTS         "\n  ts<m,c,a>[N]  Save or restore file time (modification, creation, access)"
 #define   MCHelpSwU          "\n  u             Update files"
 #define   MCHelpSwV          "\n  v             Create volumes with size autodetection or list all volumes"
 #define   MCHelpSwVn         "\n  v<size>[k,b]  Create volumes with size=<size>*1000 [*1024, *1]"
