@@ -9,8 +9,6 @@ ComprDataIO::ComprDataIO(CmdAdd *Command)
 
 void ComprDataIO::Init()
 {
-  SrcUnpack=NULL;
-  PackFromMemory=false;
   UnpackFromMemory=false;
   UnpackToMemory=false;
   UnpPackedSize=0;
@@ -19,6 +17,7 @@ void ComprDataIO::Init()
   SkipUnpCRC=false;
   PackVolume=false;
   UnpVolume=false;
+  NextVolumeMissing=false;
   SrcFile=NULL;
   DestFile=NULL;
   UnpWrSize=0;
@@ -73,7 +72,10 @@ int ComprDataIO::UnpRead(byte *Addr,uint Count)
 #ifndef NOVOLUME
       if (!MergeArchive(*SrcArc,this,true,CurrentCommand))
 #endif
+      {
+        NextVolumeMissing=true;
         return(-1);
+      }
     }
     else
       break;
@@ -110,29 +112,23 @@ void ComprDataIO::UnpWrite(byte *Addr,uint Count)
 {
 #ifdef RARDLL
   RAROptions *Cmd=((Archive *)SrcFile)->GetRAROptions();
-  if (Cmd->Callback!=NULL &&
-      Cmd->Callback(UCM_PROCESSDATA,Cmd->UserData,(LONG)Addr,Count)==-1)
-    ErrHandler.Exit(USER_BREAK);
-  if (Cmd->ProcessDataProc!=NULL)
+  if (Cmd->DllOpMode!=RAR_SKIP)
   {
-    _EBX=_ESP;
-    int RetCode=Cmd->ProcessDataProc(Addr,Count);
-    _ESP=_EBX;
-    if (RetCode==0)
+    if (Cmd->Callback!=NULL &&
+        Cmd->Callback(UCM_PROCESSDATA,Cmd->UserData,(LONG)Addr,Count)==-1)
       ErrHandler.Exit(USER_BREAK);
+    if (Cmd->ProcessDataProc!=NULL)
+    {
+      _EBX=_ESP;
+      int RetCode=Cmd->ProcessDataProc(Addr,Count);
+      _ESP=_EBX;
+      if (RetCode==0)
+        ErrHandler.Exit(USER_BREAK);
+    }
   }
 #endif
   UnpWrAddr=Addr;
   UnpWrSize=Count;
-  if (SrcUnpack!=NULL)
-  {
-    int NewSize=RepackUnpDataEnd+Count;
-    RepackUnpData.Alloc(NewSize);
-    memcpy(&RepackUnpData[RepackUnpDataEnd],Addr,Count);
-    RepackUnpDataEnd=NewSize;
-
-    SrcUnpack->SetSuspended(true);
-  }
   if (UnpackToMemory)
   {
     if (Count <= UnpackToMemorySize)
@@ -158,27 +154,13 @@ void ComprDataIO::UnpWrite(byte *Addr,uint Count)
 }
 
 
-void ComprDataIO::ShowPackRead(Int64 CurSize,Int64 UnpSize)
-{
-  if (ShowProgress)
-  {
-    Archive *DestArc=(Archive *)DestFile;
-    RAROptions *Cmd=DestArc->GetRAROptions();
-    int CurPercent=ToPercent(TotalPackRead,DestArc->AddingFilesSize);
-    if (!Cmd->DisablePercentage && CurPercent!=LastPercent)
-    {
-      mprintf("\b\b\b\b%3d%%",CurPercent);
-      LastPercent=CurPercent;
-    }
-  }
-}
 
 
 
 
 void ComprDataIO::ShowUnpRead(Int64 ArcPos,Int64 ArcSize)
 {
-  if (ShowProgress && SrcUnpack==NULL && SrcFile!=NULL)
+  if (ShowProgress && SrcFile!=NULL)
   {
     Archive *SrcArc=(Archive *)SrcFile;
     RAROptions *Cmd=SrcArc->GetRAROptions();
@@ -272,4 +254,3 @@ void ComprDataIO::SetUnpackToMemory(byte *Addr,uint Size)
   UnpackToMemorySize=Size;
 }
 
-
